/* kmp3te.cpp

  Created by SMF aka Antoine Laydier (laydier@usa.net)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */
/*=============================================================================
  HEADERs
=============================================================================*/
#include <qpopmenu.h>
#include <qmenubar.h>
#include <qpainter.h>
#include <qapp.h>
#include <qkeycode.h>
#include <qaccel.h>
#include <qpushbt.h>
#include <qdialog.h>
#include <qtimer.h>
#include <qlabel.h>
#include <qobject.h>
#include <qlistbox.h>
#include <qgrpbox.h>
#include <qevent.h>
#include <qcombo.h>
#include <qlined.h>
#include <qradiobt.h>
#include <qchkbox.h>
#include <qtabdlg.h>
#include <qtooltip.h>
#include <qmsgbox.h>
#include <qpalette.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <kconfig.h>
#include <kapp.h>
#include <klocale.h>
#include <drag.h>

#include "widgets.h"
#include "kmp3te.h"
#include "kmp3te.moc"

/*=============================================================================
  #DEFINEs
=============================================================================*/
#define ktr klocale->translate

/*=============================================================================
  GLOBALs
=============================================================================*/
KConfig       *config;
KApplication  *mykapp;

/*=============================================================================
 Class : TopLevel (methods)
=============================================================================*/

/*-----------------------------------------------------------------------------
  Routine : TopLevel::TopLevel (constructor)
-----------------------------------------------------------------------------*/
TopLevel::TopLevel( int sfolder )
         :KTopLevelWidget( )
{
  QString t;
    
  hack = FALSE;
  setMinimumSize(MIN_TE+1, 450);
  setCaption(ktr("Kde MP3 Tag Editor"));

  // Drag 'n Drop Zone...
  DNDZone = new KDNDDropZone(this,DndURL);
  connect(DNDZone,SIGNAL(dropAction(KDNDDropZone*)),this,
	  SLOT(drop(KDNDDropZone *)));
  
  // Panner ...
  panner = new KPanner(this, "panner", 
		       KPanner::U_ABSOLUTE | KPanner::O_VERTICAL, 0);
     
  //    List Box ...
  fileList = new QListBox(panner->child0(),"FileList");

  //    Tag Editor ...
  tagEditor = new TagEditor(panner->child1(), "", sfolder, fileList);
  connect(tagEditor,SIGNAL(applyButtonPressed()),this,SLOT(quitSlot()));
  
  setView(panner);
  connect(panner,SIGNAL(positionChanged()), this, SLOT(pannerMove()));

  // Menu bar ...
  file = new QPopupMenu();
  file->insertItem(ktr("&Open..."), MENU_ID_OPEN, -1);
  file->insertSeparator(-1);
  file->insertItem(ktr("&Quit"), MENU_ID_QUIT, -1);
  connect(file, SIGNAL(activated(int)), this, SLOT(menuHandler(int)));
  
  help = new QPopupMenu();
  help->insertItem(ktr("&Help"), MENU_ID_HELP, -1);
  help->insertItem(ktr("&About..."), MENU_ID_ABOUT, -1);
  connect(help, SIGNAL(activated(int)), this, SLOT(menuHandler(int)));
  
  menubar = new KMenuBar(this, "menubar");
  menubar->insertItem(ktr("&File"), file, 4, -1);
  menubar->insertSeparator(-1);
  menubar->insertItem(ktr("&Help"), help, 2, -1);
  setMenu(menubar);
  connect(menubar,SIGNAL(moved(menuPosition)), this, SLOT(menuMove()));
  
  // Read Config File ...
  t = config->readEntry(QString("G_Toplevel"));
  if( !t.isNull() &&  t.length()==24 ) { 
      int xpos,ypos,ww,wh,pan;
      sscanf(t.data(),"%04d:%04d:%04d:%04d:%04d",&xpos,&ypos,&ww,&wh,&pan);
      setGeometry(xpos,ypos,ww,wh);
      panner->setLimits(0, ww-MIN_TE);
      panner->setAbsSeparator(pan);
      resize(ww,wh);
  }
  else resize(690, 450);

  show();
  tagEditor->raiseStartUpPage();
}

/*-----------------------------------------------------------------------------
  Routine : TopLevel::~TopLevel (destructor)
-----------------------------------------------------------------------------*/
TopLevel::~TopLevel()
{
}

/*-----------------------------------------------------------------------------
  Routine : TopLevel::saveSettings(void)
-----------------------------------------------------------------------------*/
void TopLevel::saveSettings()
{
 QString  t;
 char     temp[37];
 char    *g_format = "%04d:%04d:%04d:%04d:%04d";

 sprintf( temp, g_format
        , x()     , y()
        , width() , height(), panner->getAbsSeparator() );

 config->writeEntry(QString("G_Toplevel"), QString(temp));
 config->sync();
}

/*-----------------------------------------------------------------------------
  Routine : TopLevel::closeEvent
-----------------------------------------------------------------------------*/
void  TopLevel::closeEvent( QCloseEvent * )
{
  quitSlot();
}

/*-----------------------------------------------------------------------------
  Routine : TopLevel::quitSlot()
-----------------------------------------------------------------------------*/
void TopLevel::quitSlot()
{
  tagEditor->closeFile();
  saveSettings();
  config->sync();
  qApp->quit();
}

/*-----------------------------------------------------------------------------
  Routine : TopLevel::pannerMove()
-----------------------------------------------------------------------------*/
void TopLevel::pannerMove( void )
{
  resizeEvent(NULL); // call resizeEvent without parameter
}

/*-----------------------------------------------------------------------------
  Routine : TopLevel::menuMove()
-----------------------------------------------------------------------------*/
void TopLevel::menuMove( void )
{
  resizeEvent(NULL); // call resizeEvent without parameter
}

/*-----------------------------------------------------------------------------
  Routine : TopLevel::resizeEvent
-----------------------------------------------------------------------------*/
void TopLevel::resizeEvent( QResizeEvent * ev)
{

  if(!hack) 
    panner->setLimits(0, width() - MIN_TE); // change limits of the panner
  else hack = FALSE;
  updateRects(); 
  // updates children widgets ...
  fileList->resize(panner->child0()->width(), panner->child0()->height()); 
  tagEditor->resize(panner->child1()->width(), panner->child1()->height());

}

/*-----------------------------------------------------------------------------
  Routine : TopLevel::drop
-----------------------------------------------------------------------------*/
void TopLevel::drop( KDNDDropZone * zone )
{
  unsigned int num, i;
  char *  file, * tmp;

  num = (zone->getURLList()).count();
  for(i=0;i<num;i++) {                         // process dropped objects
    tmp = (i==0 ? (zone->getURLList()).first() 
                : (zone->getURLList()).next());
    if(!strncmp(tmp,"file:",5)) {              // is this a local object ?
      file = new char[strlen(tmp)-4];
      strcpy(file, tmp+5);
      tagEditor->enqueue(file);
      delete[] file;
    }
    else {               
      QMessageBox::critical(this,
			    ktr("Error"), 
	ktr("This release of Kmp3te\nonly allow Drag N Drop of files !!!"));
    }
  }
}

/*-----------------------------------------------------------------------------
  Routine : TopLevel::menuHandler
-----------------------------------------------------------------------------*/
void TopLevel::menuHandler(int id)
{
  switch(id) {

    case MENU_ID_QUIT:
      quitSlot();
      break;

    case MENU_ID_HELP:
      mykapp->invokeHTMLHelp("","");
      break;

    case MENU_ID_ABOUT:
        QMessageBox::information(this,
			    ktr("About Kmp3TE"), 
			   "KDE MP3 Tag Editor Version "VERSION"\n\n"\
			   "Written by : 1998 SMF <smf@mail.dotcom.fr>\n");
        break;

    case MENU_ID_OPEN:
        tagEditor->enqueue(0);
        break;
        
    default:
        break;
  }
}

/*-----------------------------------------------------------------------------
  Routine : usage
-----------------------------------------------------------------------------*/
static void usage(char *name) 
{
  printf("%s [kdeopts] [--help] [-p (tag|info)]\n", name);
}

/*-----------------------------------------------------------------------------
  Routine : main
-----------------------------------------------------------------------------*/
int main( int argc, char ** argv )
{
  int i, 
  sfolder=-1;
    
  // process program  specific parameters ...
  for(i=1; i<argc; i++) {
    if( ! strcmp(argv[i],"--help") ) {
      usage(argv[0]);
      return 0;
    }
    if( strstr(argv[i],"-p") ) {
      i++;
      if ( strstr(argv[i],"tag")   ) {
	sfolder=TagEditor::PAGE_TAG;
      }
    } else if ( strstr(argv[i],"info") ) {
      sfolder=TagEditor::PAGE_INFO;
    } else {
      usage(argv[0]);
      return 1;
    }
  }
  // create a KDE application ...
  KApplication a( argc, argv, "kmp3te" );
  a.enableSessionManagement(true);
  mykapp = &a;
  config = a.getConfig(); // get configuration file
  
  // create the main window ...
  TopLevel *toplevel = new TopLevel(sfolder);
  a.setMainWidget( (QWidget *)toplevel );

  // skip the next event : resize with stange size ?
  toplevel->setHack();
  
  // run the program
  return a.exec();
}

